/*
 * Project name:
 Mini-Sub Game Demo V2
 
 * V2 Released
 9/30/2011
 * V1 Released
 9/14/2011
 
 * Created By:
 Andrew Hazelden
 * Email:
 andrewhazelden@gmail.com
 * Blog:
 http://www.andrewhazelden.com

 * Test configuration:
 MCU:             P33FJ256GP710A
 Dev.Board:       MikroMMB_for_dsPIC33_hw_rev_1.05
 http://www.mikroe.com/eng/products/view/586/mikrommb-for-dspic33-board/
 Oscillator:      64000000 Hz
 
 SW:              mikroC PRO for dsPIC
 http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 
 Controller:      mikroMedia GAMING Shield
 http://www.mikroe.com/eng/products/view/747/mikromedia-gaming-shield/
 
 MP3 Sound Chip:  VLSI VS1011E
 */

//Mini-Sub player dimensions
#define PLAYER_HEIGHT 47
#define PLAYER_WIDTH 63

//Status bar height
#define STATUS_TEXT_CELL_HEIGHT 25


#include "minisub_resources.h"
#include "minisub_setup.h"


//mikroMedia Gaming Shield Buttons
//The button controls are defined here for the dsPIC33 version
sbit upButton at PORTD.B14;
sbit downButton at PORTD.B15;
sbit rightButton at PORTD.B6;
sbit leftButton at PORTD.B9;

//Sound on / off button
sbit startButton at PORTF.B1;

//Fire button
sbit torpedoButton at PORTD.B7;   //Square Button

//This defines the status LED address
sbit LED1 at PORTA.B13;

//Count how many songs have played
extern int song_count;

//Flag to indicate that the current song is done playing
extern int play_next_song;

//Mute sound flag
extern char muteSound;


//Bonus frequency factor
int bonus_factor = 1;




void main() {
  TRISA = 0x0FFF;        // Bits 13-15 are outputs
  TRISD = 0xFFFF;        // Initialize PORTD as input
  TRISB = 0x0000;        // Initialize PORTB as output
  TRISE = 0x000;
  
  //Enable start button as an input
  TRISF.B1 = 1;
  
  LATB  = 0xFF00;       //All leds off

  //Set up the microcontroller
  Init_MCU();

  //Set up VLSI VS1011E sound playback
  file_loaded = 0;
  Init_Sound();
  MP3_Init();
  
  
  UART2_Write_Text("Mini-Sub Game Started");
  LATB  = 0x0000;      //All leds on
  

  //Fill the screen with the blue sea color
  TFT_Fill_Screen(bgColor);
  
  //Load bg music
  GetNextSong();

  //Start the main game loop
  while(1){
  
    //Reset the game
    ResetGame();

    //Show the animated game intro title
    ShowTitles();

    //Run the game until the player is out of lives
    while(subLives){
    
    //Reset the sprites
    InitSprites();
    
      //Run the current level until the sub hits a mine
      while (mineHit == 0) {

        //Check for user input
        GetInput();
        
        
        //Play the background music
        if(play_next_song){
        //Switch the current song if it is done playing
        GetNextSong();
        }
        else{
        //Play a chunk of the current song
        Play_MP3_Chunk();
        }

        //Draw the graphics
        RenderScreen();

        //Store the previous values
        SavePreviousVal();

        //Increment the frame counter
        frame_counter++;
        
      } //end of the current sub life

      //Reset the mine hit counter
      mineHit = 0;
      
      //reset the mineExplode counter
      mineExplode = 0;

      //Reduce the life counter because a submarine blew up
      subLives--;
    }

    //Display the game over title
    ShowGameOver();
  }

}





void ShowNextLevel(){
  int loop = 0;

  //Check if the user is on the next level
  if (prev_level != level){

  //Write out the current level
  UART2_Write_Text("Next Level: ");
  UART2_Write_Variable(level);
  UART2_Write(13);
  UART2_Write(10);

  //Write out the current score
  UART2_Write_Text("Score: ");
  UART2_Write_Variable(score);
  UART2_Write(13);
  UART2_Write(10);

  //Write out the current mine speed
  UART2_Write_Text("Mine Speed: ");
  UART2_Write_Variable(mine_speed);
  UART2_Write(13);
  UART2_Write(10);

  //Write out the current bonus factor
  UART2_Write_Text("Bonus distance factor: ");
  UART2_Write_Variable(bonus_factor);
  UART2_Write(13);
  UART2_Write(10);

  //Fill the screen with the blue sea color
  TFT_Fill_Screen(bgColor);

  //Display the game title
  TFT_Image(35, (SCREEN_HEIGHT/2)-47, title_bmp, 1);
  
  //Set the font color
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);

  //Convert the level from an int to string
  IntToStr(level, level_text);
  strcpy(level_display_text, "Level: ");
  strcat(level_display_text, level_text);

  //Write the score
  TFT_Write_Text(level_display_text, 140, 170);

  //Keep the sound playing
  for(loop = 0; loop<=60;loop++){
    Play_MP3_Chunk();
    Delay_ms(50);
  }


  //Fill the screen with the blue sea color
  TFT_Fill_Screen(bgColor);
  
  //Reset the sprite positions for the next level
  InitSprites();
  }


}






//Display the game over title
void ShowGameOver(){
  int loop = 0;

  UART2_Write_Line("Game Over");
  
  //Fill the screen with the blue sea color
  TFT_Fill_Screen(bgColor);
  
  //Display the game over title
  TFT_Image(0, (SCREEN_HEIGHT/2)-47, GameOver_bmp, 1);
  
  //Set the font color
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  
  
  //Check if you set a high score:
  if(score > high_score){
  //You have a high score
  high_score = score;
  TFT_Write_Text("You  set  a  High  Score!", 100, 140);
  }
  else{
  //Show the previous high score
  
  //Convert the high_score from an int to string
  IntToStr(high_score, score_text);
  strcpy(score_display_text, "High  Score: ");
  strcat(score_display_text, score_text);

  //Write the high score
  TFT_Write_Text(score_display_text, 110, 140);
  }
  
  //Convert the score from an int to string
  IntToStr(score, score_text);
  strcpy(score_display_text, "Your  Score: ");
  strcat(score_display_text, score_text);

  //Write the score
  TFT_Write_Text(score_display_text, 110, 170);

  //Keep the sound playing
  for(loop = 0; loop<=60;loop++){
    Play_MP3_Chunk();
    Delay_ms(50);
  }


  //Fill the screen with the blue sea color
  TFT_Fill_Screen(bgColor);
}


//Display the Mini-Sub title
void ShowTitles(){
  int step_value = 10;

  //Fill the screen with the blue sea color
  TFT_Fill_Screen(bgColor);

  //Set the fill brush to the sea color
  TFT_Set_Brush(1, bgColor, 0, TOP_TO_BOTTOM, CL_BLACK, CL_BLACK);

  //Display the Mini-Sub title image
  TFT_Image(35, (SCREEN_HEIGHT/2)-47, title_bmp, 1);

  //Set up the submarine y height
  sub.y = 160;

  //Animate the sub across the screen
  for(sub.x = step_value+1;sub.x <= (SCREEN_WIDTH+1);sub.x+=step_value){

    //Draw the mini-sub image
    TFT_Image(sub.x, sub.y, sub_bmp, 1);
    
    //Play background music
    Play_MP3_Chunk();


    //Clear the mini-sub
    TFT_Rectangle(sub.x-step_value, sub.y, sub.x+step_value, sub.y + PLAYER_HEIGHT );
    
    //Increment the frame counter
    frame_counter++;
  }

  //Fill the screen with the blue sea color
  TFT_Fill_Screen(bgColor);
}

void GetInput()  {
  int mine_inc = 0;
  int edge_border = 3;

  //Set user input status LED to the off value
  LED1 = 1;
  
  if(startButton){
    //UART2_Write_Line("Start");
    
    //Display the button pressed status LED
    LED1 = 0;
  }
  
  if(upButton) {
      sub.y -= SUB_Y_SPEED;
      //UART2_Write_Line("Up");

      //Display the button pressed status LED
      LED1 = 0;
    }
  else if(downButton){
      sub.y += SUB_Y_SPEED;
      //UART2_Write_Line("Down");

      //Display the button pressed status LED
      LED1 = 0;
    }
  else if(leftButton) {
      sub.x -= SUB_SPEED;
      //UART2_Write_Line("Left");

      //Display the button pressed status LED
      LED1 = 0;
    }
  else if(rightButton){
      sub.x += SUB_SPEED;
      //UART2_Write_Line("Right");

      //Display the button pressed status LED
      LED1 = 0;
    }
  else{
  
  //Only animate the sub back to the resting position every nth frame
   if( (frame_counter % 2) == 0){
      //Slowly animate the submarine back to its' default x axis resting position
      if(sub.x > DEFAULT_SUB_X_POS){
      sub.x -= 1;
      }
    }
    
    //Only animate the sub back to the resting position every nth frame
    if( (frame_counter % 4) == 0){
      //Slowly animate the submarine back to its' default x axis resting position
      if(sub.x < DEFAULT_SUB_X_POS){
      sub.x += 1;
      }
    }
  
  }
    

  //Check that the sub.y position is below the status text area
  if( sub.y <= STATUS_TEXT_CELL_HEIGHT ){
      sub.y = STATUS_TEXT_CELL_HEIGHT;
    }

  //Check that the sub.x position is greater than 0
  if( sub.x <= 0 ){
      sub.x = 0;
    }

  //Check that the sub.y position is is still onscreen
  if( sub.y >= (SCREEN_HEIGHT-PLAYER_HEIGHT)){
        sub.y = SCREEN_HEIGHT-PLAYER_HEIGHT;
    }

  //Check that the sub.x position is still onscreen
  if( sub.x >= (SCREEN_WIDTH-PLAYER_WIDTH)){
        sub.x = SCREEN_WIDTH-PLAYER_WIDTH;
    }

    //Scan the sea mine array
    for(mine_inc = 0; mine_inc <= (NUMBER_OF_SEA_MINES-1); mine_inc++){

      //Check if the mine.x value is less 0 the sea mine has gone offscreen
      if( mine[mine_inc].x <= edge_border){

      //Draw the new mine position staring on the right edge of the screen
      ResetSeaMine(mine_inc);

      //Redraw the screen
      TFT_Rectangle( mine[mine_inc].prev_x-1, mine[mine_inc].prev_y-1, mine[mine_inc].prev_x + mine[mine_inc].width + 1, mine[mine_inc].prev_y + mine[mine_inc].height + 1);

      //Increase the score after the mine goes off screen
      score += 25;
      }
      else{
      //Animate the sea mine to the left
      mine[mine_inc].x -= mine_speed;
      }

  }
  
  //Check if the bonus block x position is positive then reset its' position
  if(bonus_block.x<=edge_border){
  
  //Redraw the screen
  TFT_Rectangle( bonus_block.prev_x-1, bonus_block.prev_y-1, bonus_block.prev_x + bonus_block.width + 1, bonus_block.prev_y + bonus_block.height + 1);

  //reset the bonus block position
  ResetBonusBlock();
  }
  
  
  //animate the bonus block across the screen
  bonus_block.x -= mine_speed;



  if(torpedoButton){

    UART2_Write_Line("Fire");

    //Display the button pressed status LED
    LED1 = 0;

      if(torpedo_count>=1) {
        //Check for first fired button state and reset the torpedo position
        if( fire_torpedo_flag == 0 ) {
          //The torpedo inherits the starting position of the sub
          torpedo.x = sub.x + PLAYER_WIDTH + 1;

          //Set torpedo to launch from the center of the sub
          torpedo.y = sub.y + (PLAYER_HEIGHT/2);

          //The torpedoe visible
          torpedo.visible = 1;

          //Set the flag that the torpedo has been launched
          fire_torpedo_flag = 1;

          //Reduce the torpedo count by 1
          torpedo_count--;
          
        }
      }

    }
 
 
  //Check if there are any torpedoes remaining
  if( fire_torpedo_flag == 1 ){
     torpedo.x += TORPEDO_SPEED;

     //Check if the torpedo went off the screen
     if( torpedo.x > (SCREEN_WIDTH+TORPEDO_SPEED+1) ){
     //Hide the torpedo
     torpedo.visible = 0;
     //Reset fire_torpedo flag
     fire_torpedo_flag = 0;
     }

  }



} //end of MoveSub

//Generate a random Y height value
int RandomHeightY(){
 return (rand() % 6) * ((SCREEN_HEIGHT-STATUS_TEXT_CELL_HEIGHT)/6) + STATUS_TEXT_CELL_HEIGHT;
}


void ResetBonusBlock(){
  //reset the position of the bonus block

  //Push the bonus block 4 screens to the right
  bonus_block.x = SCREEN_WIDTH*(BONUS_SCREEN_SPACING*bonus_factor);
  //Randomize the block height
  bonus_block.y = RandomHeightY();
  
  //Add 1 to the bonus displayed counter
  bonus_counter += 1;
  
  //If the bonus is rare then make it an extra life
  
  //This random function gives a one in 3 chance for an extra life
  if( (rand() % 3) == 0){
  //Set the bonus to be a heart
   bonus_type = 2;
  }
  else {
  //Set the bonus to be a torpedo pack
  bonus_type = 1;
  }
}

void ResetSeaMine(int mineNumber){
  //Draw the new mine position staring on the right edge of the screen
  //Offset the mine Y position outside of the text area
  mine[mineNumber].x += SCREEN_WIDTH;
  mine[mineNumber].y = RandomHeightY();
}


//UART2 write text and new line (carriage return + line feed)
void UART2_Write_Line(char *uart_text) {
UART2_Write_Text(uart_text);
UART2_Write(13);
UART2_Write(10);
}

//UART2 write integer variable and new line (carriage return + line feed)
void UART2_Write_Variable(int var) {
char var_txt[12];
IntToStr(var, var_txt);
UART2_Write_Text(var_txt);
UART2_Write(13);
UART2_Write(10);
}

//UART2 write long variable and new line (carriage return + line feed)
void UART2_Write_Long_Variable(long var){
char var_txt[20];
LongWordToStr(var, var_txt);
UART2_Write_Text(var_txt);
UART2_Write(13);
UART2_Write(10);
}



//Draw the graphics onscreen
void RenderScreen(){
  int mine_inc = 0;

  //Adds a border to expand the sub redraw zone
  int redraw_border = 3;

  //Calculate the clear mine rects
  for(mine_inc = 0; mine_inc <= (NUMBER_OF_SEA_MINES-1); mine_inc++){

    clear_mine[mine_inc].top =  mine[mine_inc].prev_y;
    if(clear_mine[mine_inc].top < 0)
            clear_mine[mine_inc].top = 0;

    clear_mine[mine_inc].left =  mine[mine_inc].x + mine[mine_inc].width;
    if(clear_mine[mine_inc].left < 0)
            clear_mine[mine_inc].left = 0;

    clear_mine[mine_inc].right = mine[mine_inc].prev_x + mine[mine_inc].width;
    if(clear_mine[mine_inc].right < 0)
            clear_mine[mine_inc].right = 0;

    clear_mine[mine_inc].bottom = mine[mine_inc].prev_y + mine[mine_inc].height;
    if(clear_mine[mine_inc].bottom < 0)
            clear_mine[mine_inc].bottom = 0;

  }

  //The sub has moved to the right
  if(sub.x > sub.prev_x){
  clear_sub.left = sub.prev_x;
  clear_sub.right =  sub.x;
  clear_sub.top = sub.y;
  clear_sub.bottom = sub.y + PLAYER_HEIGHT;
  }
  
  //The sub has moved to the left
  if(sub.x < sub.prev_x){
  clear_sub.left = sub.x+PLAYER_WIDTH;
  clear_sub.right = sub.prev_x+PLAYER_WIDTH;
  clear_sub.top = sub.y;
  clear_sub.bottom = sub.y + PLAYER_HEIGHT;
  }
  
  
  //The sub has moved down
  if(sub.y > sub.prev_y){
  clear_sub.left =  sub.x;
  clear_sub.right =  sub.x+PLAYER_WIDTH;
  clear_sub.top = sub.prev_y;
  clear_sub.bottom =  sub.y;
  }

  //The sub has moved up
  if(sub.y < sub.prev_y){
  clear_sub.left =  sub.x;
  clear_sub.right =  sub.x+PLAYER_WIDTH;
  clear_sub.top = sub.y + PLAYER_HEIGHT;
  clear_sub.bottom = sub.prev_y + PLAYER_HEIGHT;
  }


  //Make sure the top left of the sub clear rect is positive
  if (clear_sub.left <=0)
    clear_sub.left = 0;

  //Keep sub clear rect below status text
  if (clear_sub.top <= STATUS_TEXT_CELL_HEIGHT)
    clear_sub.top = STATUS_TEXT_CELL_HEIGHT;

  //Set up the clear rect for the torpedo
  clear_torpedo.left = torpedo.prev_x-1;
  clear_torpedo.top =  torpedo.prev_y-1;
  clear_torpedo.right =  torpedo.x;
  clear_torpedo.bottom =  torpedo.y+torpedo.height+1;


  //Make sure the top left of the torpedo clear rect is positive
  if (clear_torpedo.left <=0)
    clear_torpedo.left = 0;

  if (clear_torpedo.top <=0)
    clear_torpedo.top = 0;


  //Set up the bonus block clear rect
  clear_bonus_block.top =  bonus_block.prev_y;
    if(clear_bonus_block.top < 0)
            clear_bonus_block.top = 0;

    clear_bonus_block.left =  bonus_block.x + bonus_block.width;
    if(clear_bonus_block.left < 0)
            clear_bonus_block.left = 0;

    clear_bonus_block.right = bonus_block.prev_x + bonus_block.width;
    if(clear_bonus_block.right < 0)
            clear_bonus_block.right = 0;

    clear_bonus_block.bottom = bonus_block.prev_y + bonus_block.height;
    if(clear_bonus_block.bottom < 0)
            clear_bonus_block.bottom = 0;



  //Fill the screen with the blue sea color
  //TFT_Fill_Screen(bgColor);

  //Set the fill brush to the sea color
  TFT_Set_Brush(1, bgColor, 0, TOP_TO_BOTTOM, CL_BLACK, CL_BLACK);

  //Only clear the sub rect if the sub has moved
  if( (sub.prev_x != sub.x)|| (sub.prev_y != sub.y) || (sub.x < (0)) || (sub.y < (STATUS_TEXT_CELL_HEIGHT)) ){
    //clear the rectangle of the old mini-sub sprite
    TFT_Rectangle(clear_sub.left, clear_sub.top, clear_sub.right, clear_sub.bottom);
  }

  //Clear the rectangle of the old sea mine sprites
  for(mine_inc = 0; mine_inc <= (NUMBER_OF_SEA_MINES-1); mine_inc++){
    TFT_Rectangle( clear_mine[mine_inc].left, clear_mine[mine_inc].top, clear_mine[mine_inc].right, clear_mine[mine_inc].bottom);
  }

  //Clear the bonus block rect if the bonus block x position is onscreen
  if(bonus_block.x <= SCREEN_WIDTH){
  //Redraw the screen
  TFT_Rectangle( clear_bonus_block.left, clear_bonus_block.top, clear_bonus_block.right, clear_bonus_block.bottom);
  }


  //Clear the rectangle of the old torpedo
  if(fire_torpedo_flag){
  //Set brush to temporarily view the clear region of the torpedo
  //TFT_Set_Brush(1, CL_BLACK, 0, TOP_TO_BOTTOM, CL_BLACK, CL_BLACK);

  TFT_Rectangle( clear_torpedo.left, clear_torpedo.top, clear_torpedo.right, clear_torpedo.bottom);
  }

  //Set the fill brush to the sea color
  TFT_Set_Brush(1, bgColor, 0, TOP_TO_BOTTOM, CL_BLACK, CL_BLACK);


  //
  //Draw the sprites onscreen
  //


  //Draw the sea mine images
  for(mine_inc = 0; mine_inc <= (NUMBER_OF_SEA_MINES-1); mine_inc++){
    TFT_Image(mine[mine_inc].x, mine[mine_inc].y, mine_bmp, 1);
  }

  //Draw the bonus block if the bonus block x position is onscreen
  if(bonus_block.x <= SCREEN_WIDTH){
      //Decide which bonus to display
      
      if(bonus_type == 2) {
      //Set the bonus to be an extra life
      TFT_Image(bonus_block.x, bonus_block.y, heart_bmp, 1);
      }
      else{
      //Set the bonus to be a torpedo pack
      TFT_Image(bonus_block.x, bonus_block.y, torpedo_pack_bmp, 1);
    }
  }

  //Draw the torpedo if it has been fired
  if(fire_torpedo_flag){
  TFT_Image(torpedo.x, torpedo.y, torpedo_bmp, 1);
  }


  //Draw the mini-sub image
  TFT_Image(sub.x, sub.y, sub_bmp, 1);
  
  
  //Debug draw the test black color to view the sub clear rect
  //TFT_Set_Brush(1, CL_BLACK, 0, TOP_TO_BOTTOM, CL_BLACK, CL_BLACK);

  //Only clear the sub rect if the sub has moved
  if( (sub.prev_x != sub.x)|| (sub.prev_y != sub.y) || (sub.x < (0)) || (sub.y < (STATUS_TEXT_CELL_HEIGHT)) ){
    //clear the rectangle of the old mini-sub sprite
    TFT_Rectangle(clear_sub.left, clear_sub.top, clear_sub.right, clear_sub.bottom);
  }


  //Scan the sea mines array to check for a mine hit by the sub
  for(mine_inc = 0; mine_inc <= (NUMBER_OF_SEA_MINES-1); mine_inc++){
    //Check for a mini-sub to sea mine collision
    if(IsCollision(sub.x, sub.y, PLAYER_WIDTH, PLAYER_HEIGHT, mine[mine_inc].x, mine[mine_inc].y, mine[mine_inc].width, mine[mine_inc].height)) {
      //Set a sea mine hit flag
      mineHit = 1;
      
      //Set the id number for the destroyed sea mine
      destroyed_mine_id = mine_inc;

      //debug which mine blew up
      UART2_Write_Text("You hit a sea mine!");
      //UART2_Write_Text("Mine hit:");
      //UART2_Write_Variable(mine_inc);
    }

    if(fire_torpedo_flag){
      //Check for a torpedo to sea mine collision
      if(IsCollision(torpedo.x, torpedo.y, torpedo.width, torpedo.height, mine[mine_inc].x, mine[mine_inc].y, mine[mine_inc].width, mine[mine_inc].height)) {
        //Set a sea mine hit flag
        mineExplode = 1;
        
        //Set the id number for the destroyed sea mine
        destroyed_mine_id = mine_inc;
        
        //Debug which mine blew up
        UART2_Write_Text("Your torpedo destroyed a sea mine!");
        //UART2_Write_Text("Mine destroyed:");
        //UART2_Write_Variable(mine_inc);
      }
    }



  }


  //Check for a mini-sub to bonus block collision
    if(IsCollision(sub.x, sub.y, PLAYER_WIDTH, PLAYER_HEIGHT, bonus_block.x, bonus_block.y, bonus_block.width, bonus_block.height)) {

      //Set the id number for the bonus
      bonus_id = 1;
      
      
      if(bonus_type == 2){
      //The sub just picked up an extra life bonus
      subLives += 1;
      
      //debug which bonus was picked up
      UART2_Write_Text("You picked up an extra life bonus!");
      }
      else{
      //The sub just picked up 3 more torpedos
      torpedo_count += 3;
      
      //debug which bonus was picked up
      UART2_Write_Text("You picked up a torpedo 3 pack bonus!");
      }

      
      //reset the bonus block position
      ResetBonusBlock();
      
      //Fill the screen with the blue sea color
      TFT_Fill_Screen(bgColor);
    }



  //Draw the fireball explosion sprite when the sub collides with a sea mine
  if(mineHit){
    //Your sub was destroyed

    //Center the fireball on the sub
    fireball.x = (sub.x + (PLAYER_WIDTH/2) ) - (fireball.width/2) ;
    fireball.y = (sub.y + (PLAYER_HEIGHT/2) ) - (fireball.height/2) ;

    //Keep the fireball from going off the top or left side of the screen
    if( fireball.x <=0)
            fireball.x=0;

    if( fireball.y <= STATUS_TEXT_CELL_HEIGHT)
            fireball.y = STATUS_TEXT_CELL_HEIGHT;

    /*
     //Debug fireball position
    UART2_Write_Text("fireball.x pos on sub fireball:");
    UART2_Write_Variable(fireball.x);

     //Debug fireball position
    UART2_Write_Text("fireball.y pos on sub fireball:");
    UART2_Write_Variable(fireball.y);

    */
    
    
    //Show fireball image
    TFT_Image(fireball.x, fireball.y, fireball_bmp, 1);

    //Fill the screen with the blue sea color
    TFT_Fill_Screen(bgColor);

  }



  //Draw the fireball explosion sprite when the torpedo blows up a sea mine
  if(mineExplode){
    //A torpedo blew up a sea mine

     //Status debugging
     //UART2_Write_Text("torpedo.x:");
     //UART2_Write_Variable(torpedo.x);
     
     //Set up the clear rect for the exploded torpedo
      clear_torpedo.left = torpedo.x;
      clear_torpedo.top =  torpedo.y;
      clear_torpedo.right =  torpedo.x+torpedo.width;
      clear_torpedo.bottom =  torpedo.y+torpedo.height;

     //Erase the torpedo from the screen
     TFT_Rectangle( clear_torpedo.left, clear_torpedo.top, clear_torpedo.right, clear_torpedo.bottom);
     
    //Center the explosion on the destroyed seamine
    explode.x = (mine[destroyed_mine_id].x + (mine[destroyed_mine_id].width/2) ) - (explode.width/2) ;
    explode.y = (mine[destroyed_mine_id].y + (mine[destroyed_mine_id].height/2) ) - (explode.height/2);

    //Keep the fireball from going off the top or left side of the screen
    if( explode.x <=0 ){
            explode.x=0;
            }

    if( explode.y <= STATUS_TEXT_CELL_HEIGHT){
            explode.y = STATUS_TEXT_CELL_HEIGHT;
            }

    //Show fireball image
    TFT_Image(explode.x, explode.y, explode_bmp, 1);
    Delay_ms(100);

    //Fill the screen with the blue sea color
    TFT_Fill_Screen(bgColor);


    //Reset mineExplode flag
    mineExplode = 0;

    //Reset torpedo launch flag
    fire_torpedo_flag = 0;

    //Reset the torpdedo after explosion
    torpedo.x = 0;
    torpedo.y = 0;
    torpedo.prev_x = 0;
    torpedo.prev_y = 0;

    //Reset the mine position after collision
    //Draw the new mine position staring on the right edge of the screen
    ResetSeaMine(destroyed_mine_id);
    mine[destroyed_mine_id].prev_x = mine[destroyed_mine_id].x;

    //Increase the score
    score += 50;
  }


  //Set the game level
  if( (level == 1) && (score > 1000) ){
  level = 2;
  prev_level = level-1;
  
  //The mine scrolling speed
  mine_speed = 8;
  
  //The bonus frequency factor
  bonus_factor = 1;
  }
  
  if( (level == 2) && (score > 2500) ){
  level = 3;
  prev_level = level-1;
  
  //The mine scrolling speed
  mine_speed = 9;
  
  //The bonus frequency factor
  bonus_factor = 2;
  }
  
  if( (level == 3) && (score > 5000) ){
  level = 4;
  prev_level = level-1;
  
  //The mine scrolling speed
  mine_speed = 10;
  
  //The bonus frequency factor
  bonus_factor = 3;
  }
  
  if( (level == 4) && (score > 8000) ){
  level = 5;
  prev_level = level-1;
  
  //The mine scrolling speed
  mine_speed = 11;
  
  //The bonus frequency factor
  bonus_factor = 5;
  }
  
  if( (level == 5) && (score > 10000) ){
  level = 6;
  prev_level = level-1;

  //The mine scrolling speed
  mine_speed = 12;

  //The bonus frequency factor
  bonus_factor = 5;
  
  //give an extra sub life
  subLives++;
  }
  
  if( (level == 6) && (score > 12000) ){
  level = 7;
  prev_level = level-1;

  //The mine scrolling speed
  mine_speed = 14;

  //The bonus frequency factor
  bonus_factor = 4;
  
  //give an extra sub life
  subLives++;
  }
  
  if( (level == 7) && (score > 15000) ){
  level = 8;
  prev_level = level-1;

  //The mine scrolling speed
  mine_speed = 14;

  //The bonus frequency factor
  bonus_factor = 4;

  //give an extra sub life
  subLives++;
  }
  
  if (prev_level != level){
  //Switch to the next level
  ShowNextLevel();
  }
  

  //Display the Score in the status bar
  RenderScore();

}


void RenderScore(){
//Display the Score in the status bar

  //Redraw the background behind the text if the values have changed
  if( (score != prev_score) ||
      (prev_torpedo_count != torpedo_count) ||
      (prev_subLives != subLives) ){

    //Debugging test region black fill color
    // TFT_Set_Brush(1, CL_BLACK, 0, TOP_TO_BOTTOM, CL_BLACK, CL_BLACK);

    //Set the fill brush to the sea color
    TFT_Set_Brush(1, bgColor, 0, TOP_TO_BOTTOM, CL_BLACK, CL_BLACK);

    //Blank the score background
    TFT_Rectangle( 0, 0, 319, 25);
  }
  
  //Draw the sub lives and the remaining torpedos icons
  
  //Torpedos remaining icon
  TFT_Image(5, 2, torpedos_remain_bmp, 1);
  
  //Lives remaining icon
  TFT_Image(270, 2, lives_bmp, 1);
  
  
  //Convert the level number from an int to string
  IntToStr(level, level_text);

  //Convert the sub lives from an int to string
  IntToStr(subLives, lives_text);

  //Convert the torpedoes from an int to string
  IntToStr(torpedo_count, torpedoes_text);

  //Convert the score from an int to string
  IntToStr(score, score_text);


  //Write the lives remaining text onscreen
  TFT_Write_Text(lives_text, (270 + 20), 2);

  //Write the torpedoes remaining text onscreen
  TFT_Write_Text(torpedoes_text, (5 + 41), 2);

  //Set up the font and font color
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);

  //Create the score string
  strcpy(score_display_text, "Score: ");
  strcat(score_display_text, score_text);
  
  //Write the score onscreen
  TFT_Write_Text(score_display_text, 120, 2);
  
  //Create the combined status stext string
  //strcat(score_display_text, "    Level: ");
  //strcat(score_display_text, level_text);
  
}


void ResetGame(){

//Reset the current level number
level = 1;
prev_level = 1;


//Reset the score
score = 0;

//Reset the starting bonus frequency factor
bonus_factor = 1;

//Reset the horizontal mine speed
mine_speed = 6;

//How many torpedos the sub starts with
torpedo_count = NUMBER_OF_TORPEDOES;

//Reset the number of submarine lives
subLives = NUMBER_OF_SUBMARINES;

//Reset the number of bonuses displayed counter
bonus_counter = 0;

//Reset the sprites
InitSprites();
}

void InitSprites(){
  int mine_inc = 0;
  
  
  //Reset the number of torpedos the sub has
  torpedo_count = NUMBER_OF_TORPEDOES;
  
  //Reset torpedo fired flag
  fire_torpedo_flag = 0;
  
  
  //Sub sprite
  sub.x = 28;
  sub.y = 128;
  sub.prev_x = sub.x;
  sub.prev_y = sub.y;
  
  
  //Sub explosion Fireball sprite
  fireball.x = 0;
  fireball.y = 0;
  fireball.prev_x = 0;
  fireball.prev_y = 0;
  fireball.width = 130;
  fireball.height = 134;
  
  
  //Sea mine explosion
  explode.x = 0;
  explode.y = 0;
  explode.prev_x = 0;
  explode.prev_y = 0;
  explode.width = 70;
  explode.height = 60;
  
  //torpedo sprite
  torpedo.x = 0;
  torpedo.y = 0;
  torpedo.prev_x = 0;
  torpedo.prev_y = 0;
  torpedo.visible = 1;
  torpedo.width = 35;
  torpedo.height = 14;
  
  //Mine sprites starting position
  for(mine_inc = 0; mine_inc <= (NUMBER_OF_SEA_MINES-1); mine_inc++){
    mine[mine_inc].x = (mine_inc * 90) + 200;
    mine[mine_inc].y = RandomHeightY();

    mine[mine_inc].prev_x = mine[mine_inc].x ;
    mine[mine_inc].prev_y = mine[mine_inc].y ;
    mine[mine_inc].width = 45;
    mine[mine_inc].height = 45;
  }
  
  
  //Torpedo bonus block sprite
  //Start the block on the 3rd screen
  bonus_block.x = SCREEN_WIDTH*BONUS_SCREEN_SPACING;
  bonus_block.y = RandomHeightY();  //generate random block Y position
  bonus_block.prev_x = 0;
  bonus_block.prev_y = 0;
  bonus_block.visible = 1;
  bonus_block.width = 45;
  bonus_block.height = 45;
  
  //set the starting bonus to be a torpedo pack
  bonus_type = 1;
  
  //Set the line drawing color
  TFT_Set_Pen(bgColor, 1);
  
  //Set the shape fill color
  TFT_Set_Brush(1, bgColor, 0, TOP_TO_BOTTOM, CL_BLACK, CL_BLACK);
  
  //Fill the screen with the blue sea color
  TFT_Fill_Screen(bgColor);
  
}

//Save the previous sprite positions
void SavePreviousVal(){
  int mine_inc = 0;
  
  //Save the previous score
  prev_score = score;
  
  //Previous sub position
  sub.prev_x = sub.x;
  sub.prev_y = sub.y;
  
  //Previous sea mine position
  for(mine_inc = 0; mine_inc <= (NUMBER_OF_SEA_MINES-1); mine_inc++){
          mine[mine_inc].prev_x = mine[mine_inc].x;
          mine[mine_inc].prev_y = mine[mine_inc].y;
  }
  
  //Previous torpedo position
  torpedo.prev_x = torpedo.x;
  torpedo.prev_y = torpedo.y;
  
  //Previous bonus block position
  bonus_block.prev_x = bonus_block.x;
  bonus_block.prev_y = bonus_block.y;
  
  //Previous values for refreshing the status screen
  prev_torpedo_count = torpedo_count;
  prev_subLives = subLives;
  
  //Reset the destroyed sea mine id number
  destroyed_mine_id = -1;
  
  //Reset the bonus id
  bonus_id = -1;
  
  //Save previous level
  prev_level = level;
  
}


//Check for a sprite to mine collision
char IsCollision (unsigned int Shape_X, unsigned int Shape_Y,  unsigned int Shape_Width, unsigned int Shape_Height,
                                  unsigned int Bomb_Left, unsigned int Bomb_Top, unsigned int Bomb_Width, unsigned int Bomb_Height) {
        
        //Check for sea mine sidewall collisions
        //The right side of Shape is greater than left side of Bomb     and        left side of Shape is smaller than right side of Bomb
        if( ((Shape_X + Shape_Width) >= Bomb_Left )               &&              ((Shape_X) <= (Bomb_Left + Bomb_Width-1)) &&
           
           
           //Check for sea mine top and bottom collisions 
           //The bottom side of Shape is greater than top of Bomb         and            top of Shape is smaller then bottom of Bomb
           ((Shape_Y + Shape_Height) >= Bomb_Top )                   &&               ((Shape_Y) <= (Bomb_Top + Bomb_Height-1))  ) {
                
                
                //Debug printing
                //UART2_Write_Line("Collision detected");
                
                //A collision has been detected
                return 1;
        }
        else
        {
                //Default return value
                //No collisions have been detected
                return 0;
        }
        
}


//Load the next song
void GetNextSong(){
 UART2_Write_Line("Switching to the next song.");

 Load_MP3_File("Pavese.mp3");
 
 /*
 if(song_count %4 == 0){
   Load_MP3_File("seyhan.mp3");
 }
 else if(song_count %4 == 1){
   Load_MP3_File("Superbia.mp3");
 }
 else if(song_count %4 == 2){
   Load_MP3_File("tmissadv.mp3");
 }
 else{
   Load_MP3_File("Pavese.mp3");
 }
 */

 //Get the MP3 header data loaded before the game continues
 if(current_pos < 10){
   PreRollSong();
 }

}

//Preroll first few bytes of the song at the startup to load the MP3 header data
void PreRollSong(){
int roll = 0;
int roll_max = 100;

//Start playing the sound
  for(roll = 0; roll<=roll_max;roll++){
    Play_MP3_Chunk();
  }
}